
#include "comb/composition-ex-colex.h"

#include "comb/comb-print.h"
#include "bits/printbin.h"
#include "demo/nextarg.h"

#include "jjassert.h"

#include "fxttypes.h"
#include "fxtio.h"


//% Compositions of n into exactly k parts in co-lexicographic (colex) order.

//#define TIMING // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 8;
    NXARG(n, "Compositions of n (n>=1)");
    ulong k = 4;
    NXARG(k, "Into exactly k parts (must have: k>=n) ");
    bool rq = 0;
    NXARG(rq, "Whether to reverse order");

    jjassert( n>=k );

    composition_ex_colex P(n, k);

    if ( rq )  P.last();
    else       P.first();

    ulong ct = 0;

#ifdef TIMING
    if ( rq )  do { ++ct; }  while  ( P.prev() != k );
    else       do { ++ct; }  while  ( P.next() != k );

#else
    const ulong *x = P.data();
    ulong q = k-1;
    do
    {
//        {  bool qq = true;
//            for (ulong j=1; j<k; ++j)  if ( abs((long)(x[j]-x[j-1])) != 1 )  { qq=false; break; }
//            if ( !qq )  --ct;  // not in OEIS
//        }
//        {  bool qq = true;
//            for (ulong j=1; j<k; ++j)  if ( x[j]-x[j-1] != 1 )  { qq=false; break; }
//            if ( !qq )  --ct;  // A129372
//        }
//        {  bool qq = true;
//            for (ulong j=1; j<k; ++j)  if ( x[j]==x[j-1] )  { qq=false; break; }
//            if ( !qq )  --ct;  // A106351
//        }
//        {  if (k>1 ) { bool qq = false;
//            for (ulong j=1; j<k; ++j)  if ( x[j]==x[j-1] )  { qq=true; break; }
//            if ( !qq )  --ct; } // A131044
//        }
//        {  bool qq = true;
//            for (ulong j=0; j<k; ++j)  if ( x[j]%2 )  { qq=false; break; }
//            if ( !qq )  --ct;  // blown up Pascal triangle
//        }
//        {  bool qq = true;
//            for (ulong j=1; j<k; ++j)  if ( ((x[j]^x[j-1])&1) !=0 )  { qq=false; break; }
//            if ( !qq )  --ct;  // (also try ==0), neither in OEIS
//        }
//        {  bool qq = true;
//            for (ulong j=0; j<k; ++j)
//                for (ulong i=j+1; i<k; ++i)
//                    if ( x[j]==x[i] )  { qq=false; goto dort; }
//        dort:
//            if ( !qq )  --ct;  // A072574, cf. A032020
//        }
//        {  bool qq = false;
//            for (ulong j=0; j<k; ++j)
//                for (ulong i=j+1; i<k; ++i)
//                    if ( x[j]==x[i] )  { qq=true; goto dort; }
//        dort:
//            if ( !qq )  --ct;  // not in OEIS
//        }


        ++ct;
        cout << setw(4) << ct << ":";
        print_vec("    ", x, k, true);
        cout << "  " << q << "  ";

        // show corresponding compositions of n-k into at most k parts:
        cout << "    [";
        for (ulong j=0; j<k; ++j)
        {
            cout << " ";
            ulong v = x[j] - 1;
            if ( 0==v )  cout << '.';
            else         cout << v;
        }
        cout << " ]";
        cout << endl;

        if ( rq )  q = P.prev();
        else       q = P.next();
    }
    while ( q != k );
#endif

    cout << "  ct=" << ct << endl;

    return 0;
}
// -------------------------


/*
Timing:

----------- next():

time ./bin 40 10 0
arg 1: 40 == n  [Compositions of n (n>=1)]  default=8
arg 2: 10 == k  [Into exactly k parts (must have: k>=n) ]  default=4
arg 3: 0 == rq  [Whether to reverse order]  default=0
  ct=211915132
./bin 40 10 0  1.05s user 0.01s system 99% cpu 1.064 total
  ==> 211915132/1.05 == 201,823,935 compositions per second
time ./bin 30 10 0

----------- prev():  identical

*/

/*
Triangle:
  for n in $(seq 1 10); do echo $(for k in $(seq 1 $n); do ./bin $n $k | grep ct= | sed 's/ct=//g'; done;); done
1
1 1
1 2 1
1 3 3 1
1 4 6 4 1
1 5 10 10 5 1
1 6 15 20 15 6 1
1 7 21 35 35 21 7 1
1 8 28 56 70 56 28 8 1
1 9 36 84 126 126 84 36 9 1

Row sums:
for n in $(seq 1 20); do echo 0 $(for k in $(seq 1 $n); do ./bin $n $k | grep ct= | sed 's/  ct=/+/g'; done;); done | bc

 */


/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/composition-ex-colex-demo.cc"
/// make-target2: "1demo DSRC=demo/comb/composition-ex-colex-demo.cc DEMOFLAGS=-DTIMING"
/// End:
